#include <windows.h>
#include <windowsx.h>
#include <time.h>
#include <math.h>
#include <fstream.h>
#include <stdio.h>
#include "..\DSP\DSP.h"
#include <conio.h>


typedef void (*Initfunc)(DWORD);
typedef void (*Uninitfunc)(void);
typedef void (*APSfunc)(float*, float*, DWORD);
typedef void (*ROFFTfunc)(float*, float*, float*, DWORD);


const float PI = 3.14159265348979f;
const float twicePI = 2.0f * PI;

int main(int c, char** argc, char** argv)
   {
   char* dspdll = "..\\DSP\\Release\\DSP.dll";
   //char* dspdll = "..\\DSP\\Debug\\DSP.dll";

   HMODULE dspmod = LoadLibrary(dspdll);
   if(dspmod==0)
      return(-1);
  
	
   Initfunc InitFFT;
   Uninitfunc UninitFFT;
   APSfunc APS;
   ROFFTfunc ROFFT;
	

	
   InitFFT = (Initfunc)GetProcAddress(dspmod, "InitFFT");
   UninitFFT = (Uninitfunc)GetProcAddress(dspmod, "UninitFFT");
   APS = (APSfunc)GetProcAddress(dspmod, "APS");
   ROFFT = (ROFFTfunc)GetProcAddress(dspmod, "ROFFT");
	

   int test;
   if(c>1)
      test = atoi(argc[1]);
   else
      test = 1;
   

   switch(test)
      {
      case 0:
         {
         long points = 8;

         if(c>2)
            points = atoi(argc[2]);

         cout<<points<<" points"<<endl;
         //float* data = new float[points*2];
         //ZeroMemory(data, points*2*sizeof(float));
         float* data = new float[points];
         float* FFTR = new float[points];
         float* FFTI = new float[points];

         const float rate = 4.0f;      //Hz
         const float dT   = 1.0f/rate; //sec
         const float freq = 0.1f;      //Hz

         cout<<"Data"<<endl;
         for(long i=0;i<points;i++)
            {
            data[i] = 10.0f * sinf( (i*dT) * (2.0f*3.1415926535f) * freq);
            cout<<i<<": "<<data[i]<<"+ 0i"<<endl;
            }

         DWORD starttime = GetTickCount();
         InitFFT(points);
         DWORD stoptime = GetTickCount();
         cout<<endl<<"Init Time (ms): "<<stoptime-starttime<<endl;

         starttime = GetTickCount();
         //for(i=0;i<10000;i++)
         ROFFT(data, FFTR, FFTI, points);
         stoptime = GetTickCount();

         cout<<endl<<"FFT Time (ms): "<<stoptime-starttime<<endl;

         cout<<"FFT"<<endl;
         for(i=0;i<points;i++)
            cout<<i<<": "<<FFTR[i]<<"+ "<<FFTI[i]<<"i"<<endl;
         
         //iFFT(data, points);

         //cout<<"iFFT"<<endl;
         //for(i=0;i<points;i++)
            //cout<<i<<": "<<FFTR[i]<<"+ "<<FFTI[i]<<"i"<<endl;
         
         UninitFFT();
         delete[] data;
         delete[] FFTR;
         delete[] FFTI;
         } break;

      case 1:
         {
         long points = 1024;

         if(c>2)
            points = atoi(argc[2]);

         cout<<points<<" points"<<endl;
         float* data = new float[points];
         float* aps = new float[points];

         const float rate = 4.0f;      //cps
         const float dT   = 1.0f/rate; //sec
         const float freq = 0.1f;      //Hz

         for(long i=0;i<points;i++)
            data[i] = 10.0f * sinf( (i*dT) * (2.0f*3.1415926535f) * freq);
         
			LARGE_INTEGER starttime, stoptime;

			QueryPerformanceCounter(&starttime);
         InitFFT(points+1);
         QueryPerformanceCounter(&stoptime);
         cout<<endl<<"Init Time (us): "<<DWORD(stoptime.QuadPart-starttime.QuadPart)<<endl;
         
         
			QueryPerformanceCounter(&starttime);
         
         long reps;
         if(c>3)
            reps = atoi(argc[3]);
         else
            reps=1;

			cout<<endl<<reps<<" reps"<<endl;
         for(i=0;i<reps;i++)
            APS(data, aps, points);
			QueryPerformanceCounter(&stoptime);
         
         cout<<"APS Time (us): "<<DWORD(stoptime.QuadPart-starttime.QuadPart)<<endl;
			cout<<float(stoptime.QuadPart-starttime.QuadPart)/reps<<" us per rep";
			cout.flush();
                  
         UninitFFT();
         delete[] data;
         delete[] aps;
         } break;      
      };

   UninitFFT();
   FreeLibrary(dspmod);
	getche();
   return(0);
   }
